/*****************************************************************************

 jcode.d:

 EUC-JP or SHIFT_JIS convert to UTF-8 for Win32

 Copyright (c) 2005 Yoshinori SUDA.

 This program is free software; you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation; either version 2 of the License, or 
 (at your option) any later version.

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 merchantability or fitness for a particular purpose. 

 See gpl.txt for more details.

 ----------------------------------------------------------------------------

 Requirement:

 - Win32

 ----------------------------------------------------------------------------

 Public Functions:

 - sjis_to_utf8
 - sjis_to_utf16
 - euc_to_utf8
 - euc_to_utf16
 - euc_to_sjis
 - isZenkaku
 - isKanji
 - isUnihan
 - isKana
 - isHiragana
 - isKatakana
 - isKatakanaPhoneticExt
 - isHalfwidthKatakana
 - isHalfwidthAndFullwidthForms

 ----------------------------------------------------------------------------

 Class Hierarchy:

 - (No Entry)

*****************************************************************************/

module suda.jcode;

private import std.utf;

extern(Windows) export int MultiByteToWideChar(
	uint	CodePage,
	uint	dwFlags,
	char*	lpMultiByteStr,
	int 	cchMultiByte,
	wchar*	lpWideCharStr,
	int 	cchWideChar);

wchar[] sjis_to_utf16(char[] s)
{
	wchar[] r;
	
	version (Windows) {
		r.length = MultiByteToWideChar(0, 0, cast(char*)s, s.length, null, 0);
		MultiByteToWideChar(0, 0, cast(char*)s, s.length, cast(wchar*)r, r.length);
	} else {
		static assert(0);
	}
	return r;
}

char[] sjis_to_utf8(char[] s)
{
	return toUTF8(sjis_to_utf16(s));
}

wchar[] euc_to_utf16(char[] s)
{
	return sjis_to_utf16(euc_to_sjis(s));
}

char[] euc_to_utf8(char[] s)
{
	return toUTF8(euc_to_utf16(s));
}

// 完全な変換では無い
// 半角カナに未対応
char[] euc_to_sjis(char[] s)
{
	char[] r;
	char[2] t;
	
	r.length = s.length;
	for (int i; i < s.length; i++) {
		if (0xA1 <= s[i] && s[i] <= 0xFE) {
			if (i + 1 >= s.length)
				break;
			t[0] = s[i]; t[1] = s[i+1];
			__euc2sjis(t);
			r[i] = t[0]; r[i+1] = t[1];
			i++;
		} else {
			r[i] = s[i];
		}
	}
	return r;
}

private void __euc2sjis(char[2] c)
{
	if ((c[0] % 2) == 0) {
		c[1] -= 0x02;
	} else {
		c[1] -= 0x61;
		if (c[1] > 0x7e)
			c[1]++;
	}
	
	c[0]++;
	c[0] /= 2;
	if (c[0] < 0xdf)
		c[0] += 0x30;
	else
		c[0] += 0x70;
}

int isZenkaku(dchar c)
{
	return isKanji(c) || isKana(c) || isHalfwidthAndFullwidthForms(c);
}

int isKanji(dchar c)
{
	return isUnihan(c);
}

int isUnihan(dchar c)
{
	return 
	   ('\U00003400' <= c && c <= '\U00004DB5')	// CJK Unified Ideographs Ext. A
	|| ('\U00004E00' <= c && c <= '\U00009FA5')	// CJK Unified Ideographs
	|| ('\U0000F900' <= c && c <= '\U0000FA2D')	// CJK Compatibility Ideographs
	|| ('\U00020000' <= c && c <= '\U0002A6D6')	// CJK Unified Ideographs Ext. B
	|| ('\U0002F800' <= c && c <= '\U0002FA1D')	// CJK C.I. Supplement
		;
}

int isKana(dchar c)
{
	return isHiragana(c) || isKatakana(c) 
		|| isKatakanaPhoneticExt(c) || isHalfwidthKatakana(c);
}

int isHiragana(dchar c)
{
	return '\U00003040' <= c && c <= '\U0000309F';
}

int isKatakana(dchar c)
{
	return '\U000030A0' <= c && c <= '\U000030FF';
}

int isKatakanaPhoneticExt(dchar c)
{
	return '\U000031F0' <= c && c <= '\U000031FF';
}

int isHalfwidthKatakana(dchar c)
{
	return '\U0000FF61' <= c && c <= '\U0000FF9F';
}

// including Fullwidth ASCII variants
int isHalfwidthAndFullwidthForms(dchar c)
{
	return '\U0000FF00' <= c && c <= '\U0000FFEF';
}
