/*****************************************************************************

 random.d:

 A Mersenne Twister Pseudorandom Number Generator

 Copyright (c) 1997-2002 Makoto Matsumoto and Takuji Nishimura.
 Copyright (c) 2005 Yoshinori SUDA.

 This D-version program modified by Yoshinori SUDA in 2005/08/15.

 ----------------------------------------------------------------------------

   A original C-program for MT19937, with initialization improved 2002/1/26.
   Coded by Takuji Nishimura and Makoto Matsumoto.
   
   Copyright (C) 1997 - 2002, Makoto Matsumoto and Takuji Nishimura,
   All rights reserved.                          
   
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:
   
     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
   
     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
   
     3. The names of its contributors may not be used to endorse or promote 
        products derived from this software without specific prior written 
        permission.
   
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   
   Any feedback is very welcome.
   http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html
   email: m-mat @ math.sci.hiroshima-u.ac.jp (remove space)

 ----------------------------------------------------------------------------

 Class Hierarchy:

 - Random

*****************************************************************************/

module suda.random;

class Random
{
private:
	const uint N = 624;
	const uint M = 397;
	
	// constant vector a most significant w-r bits least significant r bits
	const uint MATRIX_A = 0x9908b0dfU;
	const uint UPPER_MASK = 0x80000000U;
	const uint LOWER_MASK = 0x7fffffffU;
	
	uint mt[N];			// the array for the state vector
	int mti;
	
public:
	this()
	{
		seed(5489UL);	// a default initial seed is used
	}
	
	this(uint s)
	{
		seed(s);
	}
	
	this(uint[] key)
	{
		seed(key);
	}
	
	void seed(uint s)
	{
		mt[0] = s & 0xffffffffU;
		for (mti = 1; mti < N; mti++) {
			mt[mti] = 1812433253U * (mt[mti-1] ^ (mt[mti-1] >> 30)) + mti;
			// See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier.
			// In the previous versions, MSBs of the seed affect
			// only MSBs of the array mt[].
			// 2002/01/09 modified by Makoto Matsumoto
			
			mt[mti] &= 0xffffffffU;		// for >32 bit machines
		}
	}
	
	// initialize by an array with array-length 
	// init_key is the array for initializing keys 
	// key_length is its length 
	void seed(uint[] key)
	{
		int i, j, k;
		
		seed(19650218U);
		i=1; j=0;
		k = (N > key.length) ? N : key.length;
		for (; k; k--) {
			mt[i] = (mt[i] ^ ((mt[i-1] ^ (mt[i-1] >> 30)) * 1664525U))
				+ key[j] + j;		// non linear
			mt[i] &= 0xffffffffU;	// for WORDSIZE > 32 machines
			i++; j++;
			if (i >= N)
				{ mt[0] = mt[N-1]; i=1; }
			if (j >= key.length)
				j=0;
		}
		for (k=N-1; k; k--) {
			mt[i] = (mt[i] ^ ((mt[i-1] ^ (mt[i-1] >> 30)) * 1566083941U)) 
				- i;				// non linear
			mt[i] &= 0xffffffffU;	// for WORDSIZE > 32 machines
				i++;
			if (i>=N) 
				{ mt[0] = mt[N-1]; i=1; }
		}
		mt[0] = 0x80000000U; // MSB is 1; assuring non-zero initial array
	}
	
	// generates a random number on [0,0xffffffff]-interval
	uint next()
	{
    	// mag01[x] = x * MATRIX_A  for x=0,1
    	const uint[2] mag01 = [0x0U, MATRIX_A];
		
		uint y;
		
		if (mti >= N) { 	// generate N words at one time
			int kk;
			
			for (kk = 0; kk < N - M; kk++) {
				y = (mt[kk] & UPPER_MASK) | (mt[kk+1] & LOWER_MASK);
				mt[kk] = mt[kk+M] ^ (y >> 1) ^ mag01[y & 0x1UL];
			}
			for (; kk < N - 1; kk++) {
				y = (mt[kk] & UPPER_MASK) | (mt[kk+1] & LOWER_MASK);
				mt[kk] = mt[kk+(M-N)] ^ (y >> 1) ^ mag01[y & 0x1UL];
			}
			y = (mt[N-1] & UPPER_MASK) | (mt[0] & LOWER_MASK);
			mt[N-1] = mt[M-1] ^ (y >> 1) ^ mag01[y & 0x1UL];
			
			mti = 0;
		}
		y = mt[mti++];
		
		// Tempering */
		y ^= (y >> 11);
		y ^= (y << 7) & 0x9d2c5680U;
		y ^= (y << 15) & 0xefc60000U;
		y ^= (y >> 18);
		
		return y;
	}
	
	// generates a random number on [0,less)-interval
	uint next(uint less)
	{
		return cast(uint)(less * nextDouble2());
	}
	
	// generates a random number on [0,0x7fffffff]-interval
	int nextInt31()
	{
		return cast(int)(next() >> 1);
	}
	
	// generates a random number on [0,1]-real-interval
	double nextDouble()
	{
		return next() * (1.0 / 4294967295.0);
		/* divided by 2^32-1 */ 
	}
	
	// generates a random number on [0,1)-real-interval
	double nextDouble2()
	{
		return next() * (1.0 / 4294967296.0); 
		/* divided by 2^32 */
	}
	
	// generates a random number on (0,1)-real-interval
	double nextDouble3()
	{
		return (cast(double)next() + 0.5) * (1.0/4294967296.0);
		/* divided by 2^32 */
	}
	
	// generates a random number on [0,1) with 53-bit resolution
	double nextDouble53()
	{
		uint a = next() >> 5, b = next() >> 6; 
		return (a * 67108864.0 + b) * (1.0 / 9007199254740992.0);
	}
	// These real versions are due to Isaku Wada, 2002/01/09 added
}

// Global Instance
public Random random;

static this()
{
	random = new Random;
}
